@@ -1,6 +1,6 @@
 import collections
 from logging import getLogger
-from typing import TYPE_CHECKING, Dict, List, Optional
+from typing import TYPE_CHECKING, Dict, List, Optional, Tuple
 
 from pyknp import Argument as PyknpArgument
 from pyknp import Morpheme, Tag
@@ -162,6 +162,35 @@ class Argument(Component):
                 )
         return self._children
 
+    def get_constituent_base_phrases(self):
+        """The list of base phrases.
+
+        Caution:
+            When EventGraph is deserialized from a JSON file, this function becomes unavailable.
+            Consider using Python\'s pickle utility for serialization.
+        """
+        if not self.head_base_phrase:
+            logger.warning("This function is unavailable because this object is deserialized from a JSON file")
+            return
+        return self.head_base_phrase.root.modifiers(include_self=True)
+
+    @property
+    def span(self) -> List[Tuple[int, int]]:
+        spans = []
+        start_mrph_id = None
+        prev_mrph_id = None
+        for bp in filter(lambda bp: not bp.omitted_case, self.get_constituent_base_phrases()):
+            for mrph in bp.morphemes:
+                if start_mrph_id is None:
+                    start_mrph_id = mrph.mrph_id
+                if prev_mrph_id is not None and prev_mrph_id + 1 != mrph.mrph_id:
+                    spans.append((start_mrph_id, prev_mrph_id + 1))
+                    start_mrph_id = mrph.mrph_id
+                prev_mrph_id = mrph.mrph_id
+        if start_mrph_id is not None:
+            spans.append((start_mrph_id, prev_mrph_id + 1))
+        return spans
+
     def _base_phrase_to_text(
         self, bp: BasePhrase, mode: str = "mrphs", truncate: bool = False, include_modifiees: bool = False
     ) -> str:
@@ -255,14 +284,16 @@ class Argument(Component):
 
 
 class ArgumentBuilder(Builder):
-    def __call__(self, pas: "PAS", case: str, arg: PyknpArgument) -> Argument:
+    @classmethod
+    def build(cls, pas: "PAS", case: str, arg: PyknpArgument) -> Argument:
         argument = Argument(pas, case, arg.eid, arg.flag, arg.sdist, arg)
         pas.arguments[case].append(argument)
         return argument
 
 
 class JsonArgumentBuilder(Builder):
-    def __call__(self, pas: "PAS", case: str, dump: dict) -> Argument:
+    @classmethod
+    def build(cls, pas: "PAS", case: str, dump: dict) -> Argument:
         argument = Argument(pas, case, dump["eid"], dump["flag"], dump["sdist"])
         argument._surf = dump["surf"]
         argument._normalized_surf = dump["normalized_surf"]
@@ -279,19 +310,21 @@ class JsonArgumentBuilder(Builder):
 
 
 class ArgumentsBuilder(Builder):
-    def __call__(self, pas: "PAS") -> Dict[str, List[Argument]]:
+    @classmethod
+    def build(cls, pas: "PAS") -> Dict[str, List[Argument]]:
         arguments: Dict[str, List[Argument]] = collections.defaultdict(list)
         if pas.pas:
             for case, args in sorted(pas.pas.arguments.items(), key=lambda x: PAS_ORDER.get(x[0], 99)):
                 for arg in sorted(args, key=lambda _arg: (pas.ssid - _arg.sdist, _arg.tid)):
-                    arguments[case].append(ArgumentBuilder()(pas, case, arg))
+                    arguments[case].append(ArgumentBuilder.build(pas, case, arg))
         return arguments
 
 
 class JsonArgumentsBuilder(Builder):
-    def __call__(self, pas: "PAS", dump: dict) -> Dict[str, List[Argument]]:
+    @classmethod
+    def build(cls, pas: "PAS", dump: dict) -> Dict[str, List[Argument]]:
         arguments: Dict[str, List[Argument]] = collections.defaultdict(list)
         for case, arguments_dump in dump.items():
             for argument_dump in arguments_dump:
-                arguments[case].append(JsonArgumentBuilder()(pas, case, argument_dump))
+                arguments[case].append(JsonArgumentBuilder.build(pas, case, argument_dump))
         return arguments
