@@ -1,15 +1,18 @@
+import collections
 from logging import getLogger
 from typing import TYPE_CHECKING, Dict, List, Optional, Tuple, Union
 
 from pyknp import Morpheme, Tag
 
-from pyknp_eventgraph.base_phrase import BasePhrase, group_base_phrases
+from pyknp_eventgraph.base_phrase import BasePhrase
 from pyknp_eventgraph.builder import Builder
 from pyknp_eventgraph.component import Component
 from pyknp_eventgraph.features import Features, FeaturesBuilder, JsonFeaturesBuilder
 from pyknp_eventgraph.helper import PAS_ORDER, convert_katakana_to_hiragana, convert_mrphs_to_surf
 from pyknp_eventgraph.pas import PAS, JsonPASBuilder, PASBuilder
 from pyknp_eventgraph.relation import Relation
+from pyknp_eventgraph.predicate import Predicate
+from pyknp_eventgraph.argument import Argument
 
 if TYPE_CHECKING:
     from pyknp_eventgraph.sentence import Sentence
@@ -87,6 +90,16 @@ class Event(Component):
         """An alias to evid."""
         return self.evid
 
+    @property
+    def predicate(self) -> Predicate:
+        """The predicate."""
+        return self.pas.predicate
+
+    @property
+    def arguments(self) -> Dict[str, List[Argument]]:
+        """The arguments."""
+        return self.pas.arguments
+
     @property
     def surf(self) -> str:
         """A surface string."""
@@ -178,6 +191,24 @@ class Event(Component):
             self._content_rep_list = self.content_rep_list_()
         return self._content_rep_list
 
+    @property
+    def span(self) -> List[Tuple[int, int]]:
+        """The span."""
+        spans = []
+        start_mrph_id = None
+        prev_mrph_id = None
+        for bp in filter(lambda bp: not bp.omitted_case, self.get_constituent_base_phrases()):
+            for mrph in bp.morphemes:
+                if start_mrph_id is None:
+                    start_mrph_id = mrph.mrph_id
+                if prev_mrph_id is not None and prev_mrph_id + 1 != mrph.mrph_id:
+                    spans.append((start_mrph_id, prev_mrph_id + 1))
+                    start_mrph_id = mrph.mrph_id
+                prev_mrph_id = mrph.mrph_id
+        if start_mrph_id is not None:
+            spans.append((start_mrph_id, prev_mrph_id + 1))
+        return spans
+
     def surf_(self, include_modifiers: bool = False) -> str:
         """A surface string.
 
@@ -277,7 +308,7 @@ class Event(Component):
     def content_rep_list_(self) -> List[str]:
         """A list of content words."""
         content_rep_list = []
-        for bp in self._collect_base_phrases():
+        for bp in self.get_constituent_base_phrases():
             if bp.tag is None:
                 continue
             for mrph in bp.tag.mrph_list():
@@ -285,14 +316,53 @@ class Event(Component):
                     content_rep_list.append(mrph.repname or f"{mrph.midasi}/{mrph.midasi}")
         return content_rep_list
 
+    def get_constituent_base_phrases(
+        self,
+        exclude_omission: bool = False,
+        exclude_exophora: bool = False,
+    ) -> List[BasePhrase]:
+        """Collect base phrases belonging to this event.
+
+        Args:
+            exclude_omission: If true, omitted base phrases will be excluded.
+            exclude_exophora: If true, exophora will be excluded.
+
+        Caution:
+            When EventGraph is deserialized from a JSON file, this function becomes unavailable.
+            Consider using Python\'s pickle utility for serialization.
+        """
+        # Collect head base phrases.
+        if not self.pas.predicate.head_base_phrase:
+            logger.warning("This function is unavailable because this object is deserialized from a JSON file")
+            return
+
+        head_bps = [self.pas.predicate.head_base_phrase]
+        for args in self.pas.arguments.values():
+            for arg in args:
+                if arg.head_base_phrase.omitted_case:
+                    if exclude_omission and arg.flag in {"O", "E"}:
+                        # e.g., [彼が] [著者が]
+                        continue
+                    if exclude_exophora and arg.flag in {"E"}:
+                        # e.g., [著者が]
+                        continue
+                    head_bps.append(arg.head_base_phrase)
+                    continue
+                if arg.head_base_phrase.is_event_head or arg.head_base_phrase.is_event_end:
+                    continue
+                if arg.head_base_phrase.tag.tag_id > self.end.tag_id:
+                    continue
+                head_bps.append(arg.head_base_phrase)
+        return sorted(list(set(bp for head_bp in head_bps for bp in head_bp.to_list())))
+
     def _to_text(
         self,
         mode: str = "mrphs",
         truncate: bool = False,
         add_mark: bool = False,
+        exclude_omission: bool = False,
         exclude_exophora: bool = False,
         include_modifiers: bool = False,
-        exclude_adnominal: bool = False,
     ) -> str:
         """Convert this event to a text.
 
@@ -300,18 +370,17 @@ class Event(Component):
             mode: A type of token representation, which can take either "mrphs" or "reps".
             truncate: If true, adjunct words are truncated.
             add_mark: If true, special marks are added.
+            exclude_omission: If true, omitted cases will not be used.
             exclude_exophora: If true, exophora will not be used.
             include_modifiers: If true, tokens of events that modify this event will be included.
-            exclude_adnominal: If true, base phrases modified by this event will be excluded.
         """
         assert mode in {"mrphs", "reps"}
 
-        # Create a list of base phrases to show.
-        grouped_bps = group_base_phrases(
-            self._collect_base_phrases(exclude_exophora=exclude_exophora, exclude_adnominal=exclude_adnominal)
-        )
-
-        # Create a list of morphemes.
+        bps = self.get_constituent_base_phrases(exclude_omission, exclude_exophora)
+        bucket = collections.defaultdict(list)
+        for bp in sorted(bps):
+            bucket[bp.key[:-1]].append(bp)  # bp.key[-1] is the tag id.
+        grouped_bps = list(bucket.values())  # In Python 3.6+, dictionaries are insertion ordered.
         grouped_mrphs = [[morpheme for bp in bps for morpheme in bp.morphemes] for bps in grouped_bps]
 
         # Truncate the morphemes.
@@ -329,6 +398,7 @@ class Event(Component):
             normalize=truncate,
             truncated_pos=truncated_pos,
             include_modifiers=include_modifiers,
+            exclude_omission=exclude_omission,
             exclude_exophora=exclude_exophora,
         )
 
@@ -336,40 +406,6 @@ class Event(Component):
             grouped_mrphs=grouped_mrphs, mode=mode, normalize=truncate, additional_texts=additional_texts
         )
 
-    def _collect_base_phrases(
-        self,
-        exclude_exophora: bool = False,
-        exclude_adnominal: bool = False,
-    ) -> List[BasePhrase]:
-        """Collect base phrases belonging to this event.
-
-        Args:
-            exclude_exophora: If true, exophora will be excluded.
-            exclude_adnominal: If true, base phrases modified by this event will be excluded.
-
-        Returns:
-            A list of base phrases that belong to this event.
-        """
-        # Collect head base phrases.
-        head_bps = [self.pas.predicate.head_base_phrase]
-        for args in self.pas.arguments.values():
-            for arg in args:
-                if arg.head_base_phrase.omitted_case:
-                    if exclude_exophora and arg.head_base_phrase.exophora:
-                        # e.g., [著者 が]
-                        continue
-                    if exclude_adnominal and arg.head_base_phrase.tag == self.pas.predicate.head_base_phrase.tag.parent:
-                        # e.g., [車が] of "[車が] 高速道路を低速で走る" -> "車は危ない"
-                        continue
-                    head_bps.append(arg.head_base_phrase)
-                    continue
-                if arg.head_base_phrase.is_event_head or arg.head_base_phrase.is_event_end:
-                    continue
-                if arg.head_base_phrase.tag.tag_id > self.end.tag_id:
-                    continue
-                head_bps.append(arg.head_base_phrase)
-        return sorted(list(set(bp for head_bp in head_bps for bp in head_bp.to_list())))
-
     def _find_truncated_position(self, grouped_bps: List[List[BasePhrase]]) -> Tuple[int, int]:
         """Find a position just before adjunct words start.
 
@@ -426,6 +462,7 @@ class Event(Component):
         normalize: bool,
         truncated_pos: Tuple[int, int],
         include_modifiers: bool,
+        exclude_omission: bool,
         exclude_exophora: bool,
     ) -> Dict[Tuple[int, int, str], str]:
         """Get a mapping from a position to a mark.
@@ -438,6 +475,7 @@ class Event(Component):
             normalize: If true, the last content word will be normalized.
             truncated_pos: A position just before adjunct words start.
             include_modifiers: If true, tokens of events that modify this event will be included.
+            exclude_omission: If true, omitted cases will not be used.
             exclude_exophora: If true, exophora will not be used.
 
         Returns:
@@ -451,9 +489,9 @@ class Event(Component):
                     mode,
                     truncate=False,
                     add_mark=add_mark,
+                    exclude_omission=exclude_omission,
                     exclude_exophora=exclude_exophora,
                     include_modifiers=include_modifiers,
-                    exclude_adnominal=True,
                 )
                 .replace(" (", "")
                 .replace(")", "")
@@ -579,10 +617,11 @@ class Event(Component):
 
 
 class EventBuilder(Builder):
-    def __call__(self, sentence: "Sentence", start: Tag, head: Tag, end: Tag):
+    @classmethod
+    def build(cls, sentence: "Sentence", start: Tag, head: Tag, end: Tag):
         event = Event(sentence, Builder.evid, sentence.sid, sentence.ssid, start, head, end)
-        PASBuilder()(event)
-        FeaturesBuilder()(event)
+        PASBuilder.build(event)
+        FeaturesBuilder.build(event)
         sentence.events.append(event)
         Builder.evid += 1
         for tid in range(start.tag_id, end.tag_id + 1):
@@ -591,7 +630,8 @@ class EventBuilder(Builder):
 
 
 class JsonEventBuilder(Builder):
-    def __call__(self, sentence: "Sentence", dump: dict) -> Event:
+    @classmethod
+    def build(cls, sentence: "Sentence", dump: dict) -> Event:
         event = Event(sentence, Builder.evid, sentence.sid, sentence.ssid)
         event._surf = dump["surf"]
         event._surf_with_mark = dump["surf_with_mark"]
@@ -606,8 +646,8 @@ class JsonEventBuilder(Builder):
         event._normalized_reps = dump["normalized_reps"]
         event._normalized_reps_with_mark = dump["normalized_reps_with_mark"]
         event._content_rep_list = dump["content_rep_list"]
-        JsonPASBuilder()(event, dump["pas"])
-        JsonFeaturesBuilder()(event, dump["features"])
+        JsonPASBuilder.build(event, dump["pas"])
+        JsonFeaturesBuilder.build(event, dump["features"])
         sentence.events.append(event)
         Builder.evid += 1
         Builder.evid_event_map[event.evid] = event
