@@ -1,5 +1,5 @@
 from logging import getLogger
-from typing import TYPE_CHECKING, List, Optional
+from typing import TYPE_CHECKING, List, Optional, Tuple
 
 from pyknp import Morpheme, Tag
 
@@ -166,6 +166,35 @@ class Predicate(Component):
                 )
         return self._children
 
+    def get_constituent_base_phrases(self):
+        """The list of base phrases.
+
+        Caution:
+            When EventGraph is deserialized from a JSON file, this function becomes unavailable.
+            Consider using Python\'s pickle utility for serialization.
+        """
+        if not self.head_base_phrase:
+            logger.warning("This function is unavailable because this object is deserialized from a JSON file")
+            return
+        return self.head_base_phrase.root.modifiers(include_self=True)
+
+    @property
+    def span(self) -> List[Tuple[int, int]]:
+        spans = []
+        start_mrph_id = None
+        prev_mrph_id = None
+        for bp in filter(lambda bp: not bp.omitted_case, self.get_constituent_base_phrases()):
+            for mrph in bp.morphemes:
+                if start_mrph_id is None:
+                    start_mrph_id = mrph.mrph_id
+                if prev_mrph_id is not None and prev_mrph_id + 1 != mrph.mrph_id:
+                    spans.append((start_mrph_id, prev_mrph_id + 1))
+                    start_mrph_id = mrph.mrph_id
+                prev_mrph_id = mrph.mrph_id
+        if start_mrph_id is not None:
+            spans.append((start_mrph_id, prev_mrph_id + 1))
+        return spans
+
     def _base_phrase_to_text(
         self, bp: BasePhrase, mode: str = "mrphs", truncate: bool = False, include_modifiees: bool = False
     ) -> str:
@@ -255,18 +284,20 @@ class Predicate(Component):
 
 
 class PredicateBuilder(Builder):
-    def __call__(self, pas: "PAS") -> Predicate:
-        predicate = Predicate(pas, self._find_type(pas.event.head), pas.event.head)
+    @classmethod
+    def build(cls, pas: "PAS") -> Predicate:
+        predicate = Predicate(pas, cls._find_type(pas.event.head), pas.event.head)
         pas.predicate = predicate
         return predicate
 
-    @staticmethod
-    def _find_type(head: Tag) -> str:
+    @classmethod
+    def _find_type(cls, head: Tag) -> str:
         return head.features.get("用言", "")
 
 
 class JsonPredicateBuilder(Builder):
-    def __call__(self, pas: "PAS", dump: dict) -> Predicate:
+    @classmethod
+    def build(cls, pas: "PAS", dump: dict) -> Predicate:
         predicate = Predicate(pas, dump["type"])
         predicate._surf = dump["surf"]
         predicate._normalized_surf = dump["normalized_surf"]
